==> xlr-runner-module-23.1.0-217.1220-sources.jar: com/xebialabs/xlrelease/runner/impl/RunnerRegistrationResourceImpl.scala <==
package com.xebialabs.xlrelease.runner.impl

import com.xebialabs.deployit.plugin.api.reflect.Type
import com.xebialabs.xlrelease.domain.Configuration
import com.xebialabs.xlrelease.domain.runner.RemoteJobRunner
import com.xebialabs.xlrelease.repository.Ids.{CUSTOM_CONFIGURATION_ROOT, SEPARATOR}
import com.xebialabs.xlrelease.runner.api.v1.RunnerRegistrationResource
import com.xebialabs.xlrelease.runner.api.v1.RunnerRegistrationResource.{RegisterRequest, RegisterResponse}
import com.xebialabs.xlrelease.runner.service.RunnerTokenService
import com.xebialabs.xlrelease.security.PermissionChecker
import com.xebialabs.xlrelease.service.ConfigurationService
import org.springframework.stereotype.Controller

import scala.jdk.CollectionConverters._

@Controller
class RunnerRegistrationResourceImpl(permissionChecker: PermissionChecker,
                                     runnerTokenService: RunnerTokenService,
                                     configurationService: ConfigurationService)
  extends RunnerRegistrationResource {
  override def register(registerRequest: RegisterRequest): RegisterResponse = {
    permissionChecker.checkIsAllowedToRegisterRunner()
    val itemId = runnerId(registerRequest.uuid)
    val runner = if (configurationService.exists(itemId)) {
      val existingRunner = configurationService.read(itemId).asInstanceOf[RemoteJobRunner]
      if (existingRunner.getRunnerName != registerRequest.name || existingRunner.getVersion != registerRequest.version) {
        existingRunner.setRunnerName(registerRequest.name)
        existingRunner.setVersion(registerRequest.version)
        configurationService.createOrUpdate(existingRunner)
      }
      existingRunner
    } else {
      val newRunner = Type.valueOf(classOf[RemoteJobRunner]).getDescriptor.newInstance(itemId).asInstanceOf[RemoteJobRunner]
      newRunner.setTitle(registerRequest.uuid)
      newRunner.setRunnerName(registerRequest.name)
      newRunner.setVersion(registerRequest.version)
      newRunner.setCapabilities(registerRequest.capabilities.asJava)
      newRunner.setPublicKey(registerRequest.publicKey)
      configurationService.createOrUpdate(newRunner)
      newRunner
    }
    val runnerToken = runnerTokenService.createOrUpdateRunnerToken(runner.getId())
    val encryptedToken = ContextDataHelper.encryptData(runner, runnerToken.token)
    RegisterResponse(runner.getId(), runner.isEnabled(), runner.capacity, runner.encryptJobData, encryptedToken)
  }

  private def runnerId(uuid: String): String = {
    val parentId = CUSTOM_CONFIGURATION_ROOT
    val typePrefix = Type.valueOf(classOf[Configuration]).getName
    val uuidString = uuid.filterNot(_ == '-')
    s"$parentId$SEPARATOR$typePrefix$uuidString"
  }
}

==> xlr-runner-module-23.1.0-410.113-sources.jar: com/xebialabs/xlrelease/runner/impl/RunnerRegistrationResourceImpl.scala <==
package com.xebialabs.xlrelease.runner.impl

import com.xebialabs.deployit.plugin.api.reflect.Type
import com.xebialabs.xlrelease.domain.Configuration
import com.xebialabs.xlrelease.domain.runner.RemoteJobRunner
import com.xebialabs.xlrelease.domain.runner.RemoteJobRunner.DEFAULT_REMOTE_JOB_RUNNER_CAPABILITY
import com.xebialabs.xlrelease.repository.Ids.{CUSTOM_CONFIGURATION_ROOT, SEPARATOR}
import com.xebialabs.xlrelease.runner.api.v1.RunnerRegistrationResource
import com.xebialabs.xlrelease.runner.api.v1.RunnerRegistrationResource.{RegisterRequest, RegisterResponse}
import com.xebialabs.xlrelease.runner.service.RunnerTokenService
import com.xebialabs.xlrelease.scheduler.domain.LocalJobRunner.DEFAULT_LOCAL_JOB_RUNNER_CAPABILITY
import com.xebialabs.xlrelease.security.PermissionChecker
import com.xebialabs.xlrelease.service.ConfigurationService
import org.springframework.stereotype.Controller

import scala.jdk.CollectionConverters._

@Controller
class RunnerRegistrationResourceImpl(permissionChecker: PermissionChecker,
                                     runnerTokenService: RunnerTokenService,
                                     configurationService: ConfigurationService)
  extends RunnerRegistrationResource {
  override def register(registerRequest: RegisterRequest): RegisterResponse = {
    permissionChecker.checkIsAllowedToRegisterRunner()

    // All remote runner should have default capabilities as "remote". Remote runners can't have "local" as capability which is reserved for LocalRunners.
    val runnerCapabilities: Set[String] = registerRequest.capabilities ++ Set(DEFAULT_REMOTE_JOB_RUNNER_CAPABILITY) -- Set(DEFAULT_LOCAL_JOB_RUNNER_CAPABILITY)

    val itemId = runnerId(registerRequest.uuid)
    val runner = if (configurationService.exists(itemId)) {
      val existingRunner = configurationService.read(itemId).asInstanceOf[RemoteJobRunner]
      if (existingRunner.getRunnerName != registerRequest.name || existingRunner.getVersion != registerRequest.version) {
        existingRunner.setRunnerName(registerRequest.name)
        existingRunner.setVersion(registerRequest.version)
        configurationService.createOrUpdate(existingRunner)
      }
      existingRunner
    } else {
      val newRunner = Type.valueOf(classOf[RemoteJobRunner]).getDescriptor.newInstance(itemId).asInstanceOf[RemoteJobRunner]
      newRunner.setTitle(registerRequest.uuid)
      newRunner.setRunnerName(registerRequest.name)
      newRunner.setVersion(registerRequest.version)
      newRunner.setCapabilities(runnerCapabilities.asJava)
      newRunner.setPublicKey(registerRequest.publicKey)
      configurationService.createOrUpdate(newRunner)
      newRunner
    }
    val runnerToken = runnerTokenService.createOrUpdateRunnerToken(runner.getId())
    val encryptedToken = ContextDataHelper.encryptData(runner, runnerToken.token)
    RegisterResponse(runner.getId(), runner.isEnabled(), runner.capacity, runner.encryptJobData, encryptedToken)
  }

  private def runnerId(uuid: String): String = {
    val parentId = CUSTOM_CONFIGURATION_ROOT
    val typePrefix = Type.valueOf(classOf[Configuration]).getName
    val uuidString = uuid.filterNot(_ == '-')
    s"$parentId$SEPARATOR$typePrefix$uuidString"
  }
}
